#!/bin/sh

verbose=false

if [ "x$1" = "x-v" ]
then
	verbose=true
	out=/dev/stdout
	err=/dev/stderr
else
	out=/dev/null
	err=/dev/null
fi

pts=5
timeout=30

echo_n () {
	# goddamn suns can't echo -n, and Mac OS X can't echo "x\c"
	# assume argument has no doublequotes
	awk 'BEGIN { printf("'"$*"'"); }' </dev/null
}

runbochs () {
	# Find the address of the kernel readline function,
	# which the kernel monitor uses to read commands interactively.
	brkaddr=`grep 'readline$' obj/kern/kernel.sym | sed -e's/ .*$//g'`
	#echo "brkaddr $brkaddr"


	# Run Bochs, setting a breakpoint at readline(),
	# and feeding in appropriate commands to run, then quit.
	(
		echo vbreak 0x8:0x$brkaddr
		echo c
		echo die
		echo quit
	) | (
		ulimit -t $timeout
		bochs -q 'display_library: nogui' \
			'parport1: enabled=1, file="bochs.out"'
	) >$out 2>$err
}


# Usage: runtest <tagname> <defs> <strings...>
runtest() {
	perl -e "print '$1: '"
	rm -f obj/kern/init.o obj/kern/kernel obj/kern/bochs.img obj/fs/fs.img
	if $verbose
	then
		echo "gmake $2... "
	fi
	gmake $2 >$out
	if [ $? -ne 0 ]
	then
		echo gmake $2 failed 
		exit 1
	fi
	runbochs
	if [ ! -s bochs.out ]
	then
		echo 'no bochs.out'
	else
		shift
		shift
		continuetest "$@"
	fi
}

quicktest() {
	perl -e "print '$1: '"
	shift
	continuetest "$@"
}

continuetest() {
	okay=yes

	not=false
	for i
	do
		if [ "x$i" = "x!" ]
		then
			not=true
		elif $not
		then
			if egrep "^$i\$" bochs.out >/dev/null
			then
				echo "got unexpected line '$i'"
				if $verbose
				then
					exit 1
				fi
				okay=no
			fi
			not=false
		else
			egrep "^$i\$" bochs.out >/dev/null
			if [ $? -ne 0 ]
			then
				echo "missing '$i'"
				if $verbose
				then
					exit 1
				fi
				okay=no
			fi
			not=false
		fi
	done
	if [ "$okay" = "yes" ]
	then
		score=`expr $pts + $score`
		echo OK
	else
		echo WRONG
	fi
}

# Usage: runtest1 [-tag <tagname>] <progname> [-Ddef...] STRINGS...
runtest1 () {
	if [ $1 = -tag ]
	then
		shift
		tag=$1
		prog=$2
		shift
		shift
	else
		tag=$1
		prog=$1
		shift
	fi
	runtest1_defs=
	while expr "x$1" : 'x-D.*' >/dev/null; do
		runtest1_defs="DEFS+='$1' $runtest1_defs"
		shift
	done
	runtest "$tag" "DEFS='-DTEST=_binary_obj_user_${prog}_start' DEFS+='-DTESTSIZE=_binary_obj_user_${prog}_size' $runtest1_defs" "$@"
}



score=0
timeout=10

runtest1 faultread \
	! 'I read ........ from location 0!' \
	'.00000401. user fault va 00000000 ip 008.....' \
	'TRAP frame at 0xefbfffbc' \
	'  trap 0x0000000e Page Fault' \
	'  err  0x00000004' \
	'.00000401. free env 00000401'

runtest1 faultwrite \
	'.00000401. user fault va 00000000 ip 008.....' \
	'TRAP frame at 0xefbfffbc' \
	'  trap 0x0000000e Page Fault' \
	'  err  0x00000006' \
	'.00000401. free env 00000401'

runtest1 faultdie \
	'i faulted at va deadbeef, err 6' \
	'.00000401. exiting gracefully' \
	'.00000401. free env 00000401' 

runtest1 faultalloc \
	'fault deadbeef' \
	'this string was faulted in at deadbeef' \
	'fault cafebffe' \
	'fault cafec000' \
	'this string was faulted in at cafebffe' \
	'.00000401. exiting gracefully' \
	'.00000401. free env 00000401'

runtest1 faultallocbad \
	'.00000401. PFM_KILL va deadbeef ip f01.....' \
	'.00000401. free env 00000401' 

runtest1 faultnostack \
	'.00000401. PFM_KILL va eebfff.. ip f01.....' \
	'.00000401. free env 00000401'

runtest1 faultbadhandler \
	'.00000401. PFM_KILL va eebfef.. ip f01.....' \
	'.00000401. free env 00000401'

runtest1 faultevilhandler \
	'.00000401. PFM_KILL va eebfef.. ip f01.....' \
	'.00000401. free env 00000401'

runtest1 forktree \
	'....: I am .0.' \
	'....: I am .1.' \
	'....: I am .000.' \
	'....: I am .100.' \
	'....: I am .110.' \
	'....: I am .111.' \
	'....: I am .011.' \
	'....: I am .001.' \
	'.00000801. exiting gracefully' \
	'.0000040.. exiting gracefully' \
	'.0000040.. exiting gracefully' \
	'.00000c0.. exiting gracefully' \
	'.0000080.. free env 0000080.'

echo PART 1 SCORE: $score/45

pts=10
runtest1 spin \
	'.00000000. new env 00000400' \
	'.00000000. new env 00000401' \
	'I am the parent.  Forking the child...' \
	'.00000401. new env 00000402' \
	'I am the parent.  Running the child...' \
	'I am the child.  Spinning...' \
	'I am the parent.  Killing the child...' \
	'.00000401. destroying 00000402' \
	'.00000401. free env 00000402' \
	'.00000401. exiting gracefully' \
	'.00000401. free env 00000401'

runtest1 pingpong \
	'.00000000. new env 00000400' \
	'.00000000. new env 00000401' \
	'.00000401. new env 00000402' \
	'send 0 from 401 to 402' \
	'402 got 0 from 401' \
	'401 got 1 from 402' \
	'402 got 8 from 401' \
	'401 got 9 from 402' \
	'402 got 10 from 401' \
	'.00000401. exiting gracefully' \
	'.00000401. free env 00000401' \
	'.00000402. exiting gracefully' \
	'.00000402. free env 00000402' \

runtest1 primes \
	'.00000000. new env 00000400' \
	'.00000000. new env 00000401' \
	'.00000401. new env 00000402' \
	'2 .00000402. new env 00000403' \
	'3 .00000403. new env 00000404' \
	'5 .00000404. new env 00000405' \
	'7 .00000405. new env 00000406' \
	'11 .00000406. new env 00000407' 

echo PART 2 SCORE: $score/75



